/* crc.c -- not copyrighted 1994 by Greg Roelofs or Mark Adler */

#define VERSION "2.0 of 8 May 1994"


/* You can do whatever you like with this source file, though we would
   prefer that if you modify it and redistribute it that you include
   comments to that effect with your name and the date.  Thank you.

   History:
   vers     date          who           what
   ----   ---------  --------------  -----------------------------------------
    1.0    1 Mar 93  G. Roelofs      really simple 32-bit CRC filter, based on
                                      code in Mark's funzip.c, v2.3.
    1.1   15 Apr 93  G. Roelofs      added 32MB progress indicator
    2.0    8 May 94  G. Roelofs      modified for command-line arguments; made
                                      filter output buffered (37% faster);
                                      nested inner if-blocks (extra 13%); moved
                                      usage info to usage() function
 */


/*

   All crc does is take a stream of data from stdin and send it unchanged
   to stdout, calculating the CRC along the way.  The CRC and number of
   bytes read are written to stderr.

   Alternatively, if arguments are given, crc assumes that they are files
   for which the CRC is to be calculated; the file data are not written,
   and the CRC info is sent to stdout rather than stderr.

   LIKE ANYTHING ELSE WHICH IS FREE, THIS SOFTWARE COMES WITH NO WARRANTIES
   OR GUARANTEES OF ANY KIND, AND WE'RE NOT LIABLE FOR ANYTHING, SO THERE.
   (But it works for us, and our CPUs haven't melted yet.)

 */


#include "crc.h"    /* this contains the CRC data and other ugly stuff */

/* function prototypes */
void main OF((int, char **));
void usage OF((void));
void err OF((int, char *, char *));


void main(argc, argv)
    int argc;
    char **argv;
{
    register int n;                   /* data byte */
    register ulg size = 0L;           /* size of data (bytes) */
    register ulg crc = 0xffffffffL;   /* shift register contents */
    register char *outptr;            /* pointer into outbuf */
    char outbuf[OUTBUFSIZ];           /* buffer for file data */
    int filter;                       /* flag */
    FILE *in, *out;                   /* input and output files */
    FILE *msgs;                       /* where to send CRC info */


    /* skip executable name */
    --argc;
    ++argv;

#if 0   /* GRR:  add option to turn off progress indicator */
    /* get the command line option(s), if any */
    p = NULL;
    if (argc && **argv == '-') {
        --argc;
        p = 1 + *argv++;
    }
#endif

    /* if no file argument and stdin not redirected, give the user help */
    if (argc == 0) {
        if (isatty(0))
            usage();  /* (and exit) */

        /* we're a filter:  prepare to be a binary one */
        filter = TRUE;
#ifdef DOS_NT_OS2               /* some buggy C libs require BOTH the */
        setmode(0, O_BINARY);   /*  setmode() call AND the fdopen() in */
        setmode(1, O_BINARY);   /*  binary mode :-( */
#endif
        if ((in = fdopen(0, FOPR)) == NULL)
            err(2, "cannot find stdin", "");
        if ((out = fdopen(1, FOPW)) == NULL)
            err(2, "cannot write to stdout", "");
        msgs = (FILE *)stderr;
    } else {
        filter = FALSE;
        if ((in = fopen(*argv, FOPR)) == NULL)
            err(2, "cannot open ", *argv);
        --argc;
        ++argv;
        msgs = (FILE *)stdout;
    }

    /* main loop over files */
    for (;;) {
        size = 0L;
        crc = 0xffffffffL;
        outptr = outbuf;

        /* main CRC loop over file data */
        while ((n = getc(in)) != EOF) {
            ++size;
            crc = crc_32_tab[((int)crc ^ (uch)n) & 0xff] ^ (crc >> 8);
            *outptr++ = (char)n;

            /* if ((outptr-outbuf) == OUTBUFSIZ) */
            if ((size & (OUTBUFSIZ-1)) == 0) {
                if (filter)
                    fwrite(outbuf, 1, OUTBUFSIZ, out);
                outptr = outbuf;
                if ((size & 0x1ffffff) == 0) {   /* every 32MB */
                    fprintf(msgs, "   %ld MB\n", size >> 20);
                    fflush(msgs);
                }
            }
        }
        if (filter) {
            if ((outptr-outbuf) > 0)
                fwrite(outbuf, 1, (outptr-outbuf), out);
            fflush(out);
        }
        fprintf(msgs, "size = %lu bytes, CRC = %08lx\n", size, crc^0xffffffffL);
        fflush(msgs);
        if (!filter)
            fclose(in);  /* no need to do "out" (never opened) */
        if (!argc)
            break;
        if ((in = fopen(*argv, FOPR)) == NULL)
            err(2, "cannot open ", *argv);
        --argc;
        ++argv;
    }

    exit(0);

} /* end function main() */



void usage()   /* print usage and exit */
{
    fprintf(stderr, "crc, version %s, by Greg Roelofs and Mark Adler.\n",
      VERSION);
    fprintf(stderr,
      "(This software has been placed in the public domain.)\n\n");
    fprintf(stderr, "usage: ... | crc | ...\n");
    fprintf(stderr, "   or: crc file [file ...]\n\n");
    fprintf(stderr,
      "Writes stdin to stdout unchanged and computes 32-bit CRC on stderr,\n");
    fprintf(stderr,
      "or computes multiple 32-bit CRCs on stdout for given list of files.\n");
    exit(3);
}



void err(n, m, f)   /* exit on error with a message and a code */
    int n;
    char *m, *f;
{
    fprintf(stderr, "crc error: %s%s\n", m, f);
    exit(n);
}
